/*************************************************************************/
/* Title : APIC22 Asynchronous DPRAM Interface Sample Circuit            */
/*                                                                       */
/*      Language         :Verilog-HDL                                    */
/*      Module           :adpram_cnt.v                                   */
/*      Description      :Async Dual-Port RAM Controller                 */
/*                                                                       */
/*      Note : gpDPRAM̓lAKvă^C~O       */
/*             ĂB                                        */
/*                                                                       */
/*                                      ADTEK System Science Co.,Ltd.    */
/*                                      URL   :http://www.adtek.co.jp    */
/*                                      E-mail:apic22@adtek.co.jp        */
/*                                      Last Update:2001/12/10           */
/*************************************************************************/

`include "adpif_define.v"

module adpram_cnt (
			// APIC22 Local Bus Signals
			LAD,
			XLFRAME,
			M_XIO,
			R_XW,
			XLRDY,
			XLRDY_OE,
			// DPRAM Bus Signals
			DPADDR,
			XDPSEM,
			XDPCE,
			XDPOE,
			XDPWE,
			XDPBUSY,
			// System Signals
			XLRESET,
			LCLK
			) ;

/* APIC22 Local Bus Signals */
input	[`DEF_LBA-1:0]	LAD ;	// Address/Data Mux Bus
input			XLFRAME ;	// Local Frame
input			M_XIO ;		// Memory/IO Select
input			R_XW ;		// Read/Write Select
output			XLRDY ;		// In/Out Ready
output			XLRDY_OE ;

/* DPRAM Bus Signals */
output	[`DEF_DPA-1:0]	DPADDR ;	// DPRAM Address
output			XDPSEM ;	// DPRAM Semaphore Enable
output			XDPCE ;		// DPRAM Chip Enable
output			XDPOE ;		// DPRAM Data Output Enable
output			XDPWE ;		// DPRAM Data Write Enable
input			XDPBUSY ;	// DPRAM Busy Signal

/* System Signals */
input			XLRESET ;	// Local Reset
input			LCLK ;		// Local Clock


// Register Define
reg 			lrdy ;
reg 			lrdy_oe ;
reg 	[`DEF_DPA-1:0]	DPADDR ;
reg 			dpsem ;
reg 			dpce ;
reg 			dpoe ;
reg 			dpwe, dpwe_flag ;
reg 			xbusy_flag ;

reg 	[1:0]	tgt_state ;
reg 			tgt_command ;
reg 			tgt_burst ;


// Wire Define
wire			dpram_busy ;


// Parameter Define
parameter		PAR_idle  = 2'h0, 	// Bus Idle State
				PAR_addr  = 2'h1, 	// Address State
				PAR_sdata = 2'h2, 	// Single Data State
				PAR_bdata = 2'h3 ;	// Burst Data State


/*======================================*/
/* Local Bus LRDY# Equations [Begin]    */
/*======================================*/
always @(posedge LCLK or negedge XLRESET) begin
	if (!XLRESET) begin
		lrdy_oe <= 1'b0 ;
	end else begin
		lrdy_oe <= (tgt_command) ? 1'b1 : 1'b0 ;
	end
end

assign	XLRDY = (lrdy) ? 1'b0 : 1'b1 ;
assign	XLRDY_OE = (lrdy_oe) ? 1'b0 : 1'b1 ;
/*======================================*/
/* Local Bus LRDY# Equations [End]      */
/*======================================*/


/*=========================================*/
/* DPRAM Control Signals Equations [Begin] */
/*=========================================*/
assign	XDPSEM = (dpsem) ? 1'b0 : 1'b1 ;
assign	XDPCE = (dpce) ? 1'b0 : 1'b1 ;
assign	XDPOE = (dpoe) ? 1'b0 : 1'b1 ;
/*=========================================*/
/* DPRAM Control Signals Equations [End]   */
/*=========================================*/


/*======================================*/
/* DPRAM Busy Signal Equations [Begin]  */
/*======================================*/
// Wire Define
wire		busy_flag_clr = (!XLRESET || !XDPBUSY) ? 1'b1 : 1'b0 ;

always @(posedge LCLK or posedge busy_flag_clr) begin
	if (busy_flag_clr) begin
		xbusy_flag <= 1'b0 ;
	end else begin
		xbusy_flag <= (XDPBUSY) ? 1'b1 : 1'b0 ;
	end
end

assign	dpram_busy = (!XDPBUSY || !xbusy_flag) ? 1'b1 : 1'b0 ;
/*======================================*/
/* DPRAM Busy Signal Equations [End]    */
/*======================================*/


/*======================================*/
/* DPRAM Write Enable Equations [Begin] */
/*======================================*/
//            ___     ___     ___    
// LCLK          |___|   |___|   |___
//                    _______        
// dpwe       _______|       |_______
//                        _______    
// dpwe_flag  ___________|       |___
//            _______     ___________
// XDPWE             |___|           
always @(negedge LCLK or negedge XLRESET) begin
	if (!XLRESET) begin
		dpwe_flag <= 1'b0 ;
	end else begin
		dpwe_flag <= (dpwe) ? 1'b1 : 1'b0 ;
	end
end

assign	XDPWE = (dpwe && !dpwe_flag) ? 1'b0 : 1'b1 ;
/*======================================*/
/* DPRAM Write Enable Equations [End]   */
/*======================================*/


/*===========================================*/
/* DPRAM Control Sequencer Equations [Begin] */
/*===========================================*/
always @(posedge LCLK or negedge XLRESET) begin
	if (!XLRESET) begin
		tgt_state <= 2'h0 ;
		tgt_command <= 1'b0 ;
		tgt_burst <= 1'b0 ;
		lrdy <= 1'b0 ;
		DPADDR <= {`DEF_DPA{1'b0}} ;
		dpsem <= 1'b0 ;
		dpce <= 1'b0 ;
		dpoe <= 1'b0 ;
		dpwe <= 1'b0 ;
	end else begin
		case (tgt_state)
			PAR_idle	: begin	// Bus Idle State
							if (!XLFRAME && !tgt_command) begin
								tgt_state <= PAR_addr ;
								tgt_command <= 1'b1 ;
							end else begin
								tgt_state <= PAR_idle ;
								tgt_command <= 1'b0 ;
							end
							tgt_burst <= 1'b0 ;
							lrdy <= 1'b0 ;
							DPADDR <= LAD[`DEF_DPA+1:2] ;
							dpsem <= 1'b0 ;
							dpce <= 1'b0 ;
							dpoe <= 1'b0 ;
							dpwe <= 1'b0 ;
						  end
			PAR_addr	: begin	// Address State
							if (!XLFRAME) begin
								tgt_state <= PAR_bdata ;
								tgt_burst <= 1'b1 ;
							end else begin
								tgt_state <= PAR_sdata ;
								tgt_burst <= 1'b0 ;
							end
							tgt_command <= 1'b1 ;
							lrdy <= 1'b0 ;
							DPADDR <= (tgt_burst) ? DPADDR + 1 : DPADDR ;
							dpsem <= (!M_XIO) ? 1'b1 : 1'b0 ;
							dpce <= (M_XIO) ? 1'b1 : 1'b0 ;
							dpoe <= 1'b0 ;
							dpwe <= 1'b0 ;
						   end
			PAR_sdata	: begin	// Single Data State
							if (dpram_busy) begin
								tgt_state <= PAR_sdata ;
								dpoe <= 1'b0 ;
								dpwe <= 1'b0 ;
								lrdy <= 1'b0 ;
							end else begin
								tgt_state <= PAR_idle ;
								dpoe <= (R_XW) ? 1'b1 : 1'b0 ;
								dpwe <= (!R_XW) ? 1'b1 : 1'b0 ;
								lrdy <= 1'b1 ;
							end
							tgt_command <= 1'b1 ;
							tgt_burst <= 1'b0 ;
							DPADDR <= DPADDR ;
							dpsem <= dpsem ;
							dpce <= dpce ;
						  end
			PAR_bdata	: begin	// Burst Data State
							if (dpram_busy) begin
								tgt_state <= PAR_bdata ;
								dpoe <= 1'b0 ;
								dpwe <= 1'b0 ;
								lrdy <= 1'b0 ;
							end else begin
								if (!XLFRAME) begin
									tgt_state <= PAR_addr ;
								end else begin
									tgt_state <= PAR_idle ;
								end
								dpoe <= (R_XW) ? 1'b1 : 1'b0 ;
								dpwe <= (!R_XW) ? 1'b1 : 1'b0 ;
								lrdy <= 1'b1 ;
							end
							tgt_command <= 1'b1 ;
							tgt_burst <= 1'b1 ;
							DPADDR <= DPADDR ;
							dpsem <= dpsem ;
							dpce <= dpce ;
						  end
		endcase
	end
end
/*===========================================*/
/* DPRAM Control Sequencer Equations [End]   */
/*===========================================*/


endmodule
