/*************************************************************************/
/* Title : APIC22 Synchronous DPRAM Interface Sample Circuit             */
/*                                                                       */
/*      Language         :Verilog-HDL                                    */
/*      Module           :sdpram_cnt.v                                   */
/*      Description      :Sync Dual-Port RAM Controller                  */
/*                                                                       */
/*      Note : gpDPRAM̓lAKvă^C~O       */
/*             ĂB                                        */
/*                                                                       */
/*                                      ADTEK System Science Co.,Ltd.    */
/*                                      URL   :http://www.adtek.co.jp    */
/*                                      E-mail:apic22@adtek.co.jp        */
/*                                      Last Update:2001/12/10           */
/*************************************************************************/

module sdpram_cnt (
			// APIC22 Local Bus Signals
			XBE,
			XLFRAME,
			M_XIO,
			R_XW,
			XLRDY,
			XLRDY_OE,
			// DPRAM Bus Signals
			XDPBE,
			XDPCE,
			XDPOE,
			XDPWE,
			XDPACEN,
			// System Signals
			XLRESET,
			LCLK
			) ;

/* APIC22 Local Bus Signals */
input	[3:0]	XBE ;		// Local Byte Enable
input			XLFRAME ;	// Local Frame
input			M_XIO ;		// Memory/IO Select
input			R_XW ;		// Read/Write Select
output			XLRDY ;		// In/Out Ready
output			XLRDY_OE ;

/* DPRAM Bus Signals */
output	[3:0]	XDPBE ;		// DPRAM Byte Enable
output			XDPCE ;		// DPRAM Chip Enable
output			XDPOE ;		// DPRAM Data Output Enable
output			XDPWE ;		// DPRAM Data Write Enable
output			XDPACEN ;	// DPRAM Busy Signal

/* System Signals */
input			XLRESET ;	// Local Reset
input			LCLK ;		// Local Clock


// Register Define
reg 			lrdy ;
reg 			lrdy_oe ;
reg 	[3:0]	XDPBE ;
reg 			dpce ;
reg 			dpoe ;
reg 			dpwe ;

reg 	[1:0]	tgt_state ;
reg 			tgt_command ;
reg 			tgt_burst ;


// Parameter Define
parameter		PAR_idle  = 2'h0, 	// Bus Idle State
				PAR_addr  = 2'h1, 	// Address State
				PAR_sdata = 2'h2, 	// Single Data State
				PAR_bdata = 2'h3 ;	// Burst Data State


/*======================================*/
/* Local Bus LRDY# Equations [Begin]    */
/*======================================*/
always @(posedge LCLK or negedge XLRESET) begin
	if (!XLRESET) begin
		lrdy_oe <= 1'b0 ;
	end else begin
		lrdy_oe <= ( (!XLFRAME && M_XIO && !tgt_command)
		             || (tgt_command && (tgt_state != PAR_idle))
		           ) ? 1'b1 : 1'b0 ;
	end
end

assign	XLRDY = (lrdy) ? 1'b0 : 1'b1 ;
assign	XLRDY_OE = (lrdy_oe) ? 1'b0 : 1'b1 ;
/*======================================*/
/* Local Bus LRDY# Equations [End]      */
/*======================================*/


/*======================================*/
/* DPRAM Byte Enable Equations [Begin]  */
/*======================================*/
always @(posedge LCLK or negedge XLRESET) begin
	if (!XLRESET) begin
		XDPBE <= 4'b1111 ;
	end else begin
		if (tgt_command) begin
			XDPBE <= (R_XW) ? 4'b0000 : XBE[3:0] ;
		end else begin
			XDPBE <= 4'b1111 ;
		end
	end
end
/*======================================*/
/* DPRAM Byte Enable Equations [End]    */
/*======================================*/


/*=========================================*/
/* DPRAM Control Signals Equations [Begin] */
/*=========================================*/
assign	XDPCE = (dpce) ? 1'b0 : 1'b1 ;
assign	XDPOE = (dpoe) ? 1'b0 : 1'b1 ;
assign	XDPWE = (dpwe) ? 1'b0 : 1'b1 ;
assign	XDPACEN = (tgt_burst) ? 1'b0 : 1'b1 ;
/*=========================================*/
/* DPRAM Control Signals Equations [End]   */
/*=========================================*/


/*===========================================*/
/* DPRAM Control Sequencer Equations [Begin] */
/*===========================================*/
always @(posedge LCLK or negedge XLRESET) begin
	if (!XLRESET) begin
		tgt_state <= 2'h0 ;
		tgt_command <= 1'b0 ;
		tgt_burst <= 1'b0 ;
		lrdy <= 1'b0 ;
		dpce <= 1'b0 ;
		dpoe <= 1'b0 ;
		dpwe <= 1'b0 ;
	end else begin
		case (tgt_state)
			PAR_idle	: begin	// Bus Idle State
							if (!XLFRAME && M_XIO && !tgt_command) begin
								tgt_state <= PAR_addr ;
								tgt_command <= 1'b1 ;
								dpce <= 1'b1 ;
							end else begin
								tgt_state <= PAR_idle ;
								tgt_command <= 1'b0 ;
								dpce <= 1'b0 ;
							end
							tgt_burst <= 1'b0 ;
							lrdy <= 1'b0 ;
							dpoe <= 1'b0 ;
							dpwe <= 1'b0 ;
						  end
			PAR_addr	: begin	// Address State
							if (!XLFRAME) begin
								tgt_state <= PAR_bdata ;
								tgt_burst <= (R_XW) ? 1'b1 : 1'b0 ;
							end else begin
								tgt_state <= PAR_sdata ;
								tgt_burst <= 1'b0 ;
							end
							tgt_command <= 1'b1 ;
							lrdy <= 1'b1 ;
							dpce <= 1'b1 ;
							dpoe <= 1'b0 ;
							dpwe <= 1'b0 ;
						   end
			PAR_sdata	: begin	// Single Data State
							tgt_state <= PAR_idle ;
							tgt_command <= 1'b1 ;
							tgt_burst <= 1'b0 ;
							lrdy <= 1'b0 ;
							dpce <= 1'b1 ;
							dpoe <= (R_XW) ? 1'b1 : 1'b0 ;
							dpwe <= (!R_XW) ? 1'b1 : 1'b0 ;
						  end
			PAR_bdata	: begin	// Burst Data State
							if (!XLFRAME) begin
								tgt_state <= PAR_bdata ;
								tgt_command <= 1'b1 ;
								tgt_burst <= (R_XW || dpwe) ? 1'b1 : 1'b0 ;
								lrdy <= 1'b1 ;
								dpce <= 1'b1 ;
								dpoe <= (R_XW) ? 1'b1 : 1'b0 ;
								dpwe <= (!R_XW) ? 1'b1 : 1'b0 ;
							end else begin
								tgt_state <= PAR_idle ;
								tgt_command <= 1'b0 ;
								tgt_burst <= 1'b0 ;
								lrdy <= 1'b0 ;
								dpce <= 1'b0 ;
								dpoe <= 1'b0 ;
								dpwe <= 1'b0 ;
							end
						  end
		endcase
	end
end
/*===========================================*/
/* DPRAM Control Sequencer Equations [End]   */
/*===========================================*/


endmodule
